﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Configuration;
using System.Collections.Generic;
using System.IO;
using Advania.Signet.Cloud2Connector.Models;

namespace Advania.Signet.Cloud2Connector.Testing
{
    [TestClass]
    public class Cloud2Tests
    {
        Connector connector = new Connector(ConfigurationManager.AppSettings.Get("CloudUrl"),
            ConfigurationManager.AppSettings.Get("CloudCert"), ConfigurationManager.AppSettings.Get("CloudUser"),
            ConfigurationManager.AppSettings.Get("CloudPassword"));
        string ssn = ConfigurationManager.AppSettings.Get("SSN");
        string mobile = ConfigurationManager.AppSettings.Get("Mobile");
        string id = ConfigurationManager.AppSettings.Get("CloudID");

        [TestMethod]
        public void TestPing()
        {
            string res = null;
            try
            {
                res = connector.Ping("Testing");
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestAddDocument()
        {
            AddDocumentResponse res = null;
            try
            {
                FieldLocation location = new FieldLocation { Height = 90, Width = 190, Page = 1, XLocation = 200, YLocation = 198 };
                string baseDir = AppDomain.CurrentDomain.BaseDirectory;
                byte[] doc = File.ReadAllBytes(baseDir.Substring(0, baseDir.LastIndexOf("\\bin\\")) + "\\Resources\\Viðauki 1 - Dæmi um skjal til undirritunar í kerfum bjóðanda.pdf");
                res = connector.AddDocuments(new List<Document>(new Document[] {new Document { Data = doc, DocumentName = "Testing.pdf" }}),
                    "Testing", "signatureAppearance=6;", "Testing", new List<DocumentSigner>(new DocumentSigner[] { new DocumentSigner { Order = 1, SSN = ssn, Location=location } }),
                     false, true, null, null, null);
                id = res.DocIDs[0];
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestDownloadDocument()
        {
            List<Document> res = null;
            try
            {                
                res = connector.DownloadDocument(id, null);                
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestSignDocument()
        {
            bool res = false;
            try
            {
                res = connector.SignDocument(id, null, ssn, mobile);
                Assert.IsTrue(res);
            }
            catch (Exception ex)
            {
                Assert.IsTrue(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestGetDocumentInfo()
        {
            List<DocumentInfo> res = null;
            try
            {
                res = connector.GetDocumentInfo(id, null);
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestGetDocuments()
        {
            List<DocumentInfo> res = null;
            try
            {
                res = connector.GetDocuments(ssn, null, null, null);
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestDeleteDocument()
        {
            bool res = false;
            try
            {
                res = connector.DeleteDocument(id, null);
                Assert.IsTrue(res);
            }
            catch (Exception ex)
            {
                Assert.IsTrue(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestGetNotification()
        {
            ResultStatus res = null;
            try
            {
                res = connector.GetNotification(id, null);
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestRemindDocument()
        {            
            try
            {
                connector.RemindDocument(id, null);
                Assert.IsTrue(true);
            }
            catch (Exception ex)
            {
                Assert.IsTrue(false, ex.Message);
            }
        }

        [TestMethod]
        public void TestGetToken()
        {
            string res = null;
            try
            {
                res = connector.GetToken(id, null, ssn, mobile, "https://www.signet.is", null, null, null);
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }
    }
}
