﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Configuration;
using System.Collections.Generic;
using System.IO;
using Advania.Signet.Cloud2Connector.Models;

namespace Advania.Signet.Cloud2Connector.Testing
{
    [TestClass]
    public class Cloud2TokenTests
    {
        TokenConnector connector = new TokenConnector(ConfigurationManager.AppSettings.Get("CloudTokenUrl"),
            ConfigurationManager.AppSettings.Get("CloudCert"), ConfigurationManager.AppSettings.Get("CloudUser"),
            ConfigurationManager.AppSettings.Get("CloudPassword"), "https://signet.is");
        string ssn = ConfigurationManager.AppSettings.Get("SSN");
        string mobile = ConfigurationManager.AppSettings.Get("Mobile");
        string id = ConfigurationManager.AppSettings.Get("CloudID");
        static string baseDir = AppDomain.CurrentDomain.BaseDirectory;
        static string resourceDir = baseDir.Substring(0, baseDir.LastIndexOf("\\bin\\")) + "\\Resources\\";
        string token = File.Exists(resourceDir + "token.txt") ? File.ReadAllText(resourceDir + "token.txt") : null;        
       
        [TestMethod]
        public void TestAddDocument()
        {
            AddDocumentResponse res = null;
            try
            {                
                byte[] doc = File.ReadAllBytes(resourceDir + "LoremIpsum.pdf");
                TokenInit();
                res = connector.AddDocuments(new List<Document>(new Document[] {new Document { Data = doc, DocumentName = "Testing.pdf" }}),
                    "Testing", "Testing", "Testing", new List<DocumentSigner>(new DocumentSigner[] { new DocumentSigner { Order = 1, SSN = ssn } }),
                     false, null, null, null);
                id = res.DocIDs[0];
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestDownloadDocument()
        {
            List<Document> res = null;
            try
            {
                TokenInit();
                res = connector.DownloadDocument(id, null);                
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestGetDocumentInfo()
        {
            List<DocumentInfo> res = null;
            try
            {
                TokenInit();
                res = connector.GetDocumentInfo(id, null);
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestGetDocuments()
        {
            List<DocumentInfo> res = null;
            try
            {
                TokenInit();
                res = connector.GetDocuments();
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestDeleteDocument()
        {
            bool res = false;
            try
            {
                TokenInit();
                res = connector.DeleteDocument(id, null);
                Assert.IsTrue(res);
            }
            catch (Exception ex)
            {
                Assert.IsTrue(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestSignDocument()
        {
            bool res = false;
            try
            {
                TokenInit();
                res = connector.SignDocument(id, null, null);
                Assert.IsTrue(res);
            }
            catch (Exception ex)
            {
                Assert.IsTrue(res, ex.Message);
            }
        }

        [TestMethod]
        public void TestRejectDocument()
        {
            bool res = false;
            try
            {
                TokenInit();
                res = connector.RejectDocument(id, null, "Test", null, true);
                Assert.IsTrue(res);
            }
            catch (Exception ex)
            {
                Assert.IsTrue(res, ex.Message);
            }
        }
      
        [TestMethod]
        public void TestGetSignToken()
        {
            string res = null;
            try
            {
                TokenInit();
                res = connector.GetSignToken(id, null, "https://www.signet.is", null);
                Assert.IsNotNull(res);
            }
            catch (Exception ex)
            {
                Assert.IsNotNull(res, ex.Message);
            }
        }

        private void TokenInit()
        {
            if (!string.IsNullOrWhiteSpace(token))
                connector.SetToken(token, ssn, mobile);
            else
            {
                token = connector.GetToken(ssn, mobile);
                File.WriteAllText(resourceDir + "token.txt", token);
            }
        }
    }
}
