﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Advania.Signet.Cloud2Connector
{
    public class CommonHelper
    {
        public static string type = "application/json";
        private static byte[] key = { };
        private static byte[] IV = { 0x12, 0x34, 0x56, 0x78, 0x90, 0xAB, 0xCD, 0xEF };
        private static string DefaultEncryptionKey = ConfigurationManager.AppSettings.Get("DefaultEncryptionKey") ?? "qB7RSWXGXbtC4Z4BcKb6gyUn";

        /// <summary>
        /// Post data
        /// </summary>
        /// <param name="data"></param>
        /// <param name="url"></param>
        /// <param name="cert"></param>
        /// <returns></returns>
        public static string PostData(string data, string url, X509Certificate2 cert, bool put = false, string apikey = null)
        {
            WebRequestHandler certhandler = new WebRequestHandler();
            if (cert != null)
            {
                certhandler.ClientCertificateOptions = ClientCertificateOption.Manual;
                certhandler.ClientCertificates.Add(cert);
            }

            using (var content = new StringContent(data, Encoding.UTF8, type))
            using (var httpClient = new HttpClient(certhandler))
            {
                httpClient.DefaultRequestHeaders.Accept.Clear();
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(type));
                if (!string.IsNullOrEmpty(apikey))
                    httpClient.DefaultRequestHeaders.Add("apikey", apikey);
                var response = put ? httpClient.PutAsync(url, content).Result : httpClient.PostAsync(url, content).Result;
                if (!response.IsSuccessStatusCode)
                {
                    var errorResponse = response.ReasonPhrase;
                    string result = response.Content.ReadAsStringAsync().Result;
                    throw new Exception(String.Format("Exception when calling service: {0} {1}", url, errorResponse + result));
                }
                return response.Content.ReadAsStringAsync().Result;
            }
        }

        /// <summary>
        /// Post data
        /// </summary>
        /// <param name="data"></param>
        /// <param name="url"></param>
        /// <param name="user"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public static string PostData(string data, string url, string user, string password, bool put = false, string apikey = null)
        {
            HttpClientHandler handler = new HttpClientHandler();
            CredentialCache credentials = new CredentialCache();
            credentials.Add(new System.Uri(url), "Basic", new NetworkCredential(user, password));
            handler.Credentials = credentials;
            handler.PreAuthenticate = true;

            using (var content = new StringContent(data, Encoding.UTF8, type))
            using (var httpClient = new HttpClient(handler))
            {
                httpClient.DefaultRequestHeaders.Accept.Clear();
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(type));
                httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic",
                            Convert.ToBase64String(System.Text.Encoding.UTF8.GetBytes(user + ":" + password)));
                if (!string.IsNullOrEmpty(apikey))
                    httpClient.DefaultRequestHeaders.Add("apikey", apikey);
                var response = put ? httpClient.PutAsync(url, content).Result : httpClient.PostAsync(url, content).Result;
                if (!response.IsSuccessStatusCode)
                {
                    var errorResponse = response.ReasonPhrase;
                    string result = response.Content.ReadAsStringAsync().Result;
                    throw new Exception(String.Format("Exception when calling service: {0} {1}", url, errorResponse + result));
                }
                return response.Content.ReadAsStringAsync().Result;
            }
        }

        /// <summary>
        /// Get data
        /// </summary>        
        /// <param name="url"></param>
        /// <param name="cert"></param>
        /// <returns></returns>
        public static string GetData(string url, X509Certificate2 cert, bool delete = false)
        {
            WebRequestHandler certhandler = new WebRequestHandler();
            certhandler.ClientCertificateOptions = ClientCertificateOption.Manual;
            certhandler.ClientCertificates.Add(cert);

            using (var httpClient = new HttpClient(certhandler))
            {
                httpClient.DefaultRequestHeaders.Accept.Clear();
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(type));
                var response = delete ? httpClient.DeleteAsync(url).Result : httpClient.GetAsync(url).Result;
                if (!response.IsSuccessStatusCode)
                {
                    var errorResponse = response.ReasonPhrase;
                    string result = response.Content.ReadAsStringAsync().Result;
                    throw new Exception(String.Format("Exception when calling service: {0} {1}", url, errorResponse + result));
                }
                return response.Content.ReadAsStringAsync().Result;
            }
        }

        /// <summary>
        /// Get file data
        /// </summary>        
        /// <param name="url"></param>
        /// <param name="cert"></param>
        /// <returns></returns>
        public static byte[] GetData(string url, X509Certificate2 cert, out string filename)
        {
            WebRequestHandler certhandler = new WebRequestHandler();
            certhandler.ClientCertificateOptions = ClientCertificateOption.Manual;
            certhandler.ClientCertificates.Add(cert);

            using (var httpClient = new HttpClient(certhandler))
            {
                httpClient.DefaultRequestHeaders.Accept.Clear();
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(type));
                var response = httpClient.GetAsync(url);
                if (!response.Result.IsSuccessStatusCode)
                {
                    var errorResponse = response.Result.ReasonPhrase;
                    throw new Exception(String.Format("Exception when calling service: {0} {1}", url, errorResponse));
                }
                else
                {
                    filename = response.Result.Content.Headers.ContentDisposition.FileName;
                    return response.Result.Content.ReadAsByteArrayAsync().Result;
                }
            }
        }

        /// <summary>
        /// Get data
        /// </summary>        
        /// <param name="url"></param>
        /// <param name="cert"></param>
        /// <returns></returns>
        public static string GetData(string url, string user, string password, bool delete = false)
        {
            WebRequestHandler handler = new WebRequestHandler();
            CredentialCache credentials = new CredentialCache();
            credentials.Add(new System.Uri(url), "Basic", new NetworkCredential(user, password));
            handler.Credentials = credentials;
            handler.PreAuthenticate = true;

            using (var httpClient = new HttpClient(handler))
            {
                httpClient.DefaultRequestHeaders.Accept.Clear();
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(type));
                var response = delete ? httpClient.DeleteAsync(url).Result : httpClient.GetAsync(url).Result;
                if (!response.IsSuccessStatusCode)
                {
                    var errorResponse = response.ReasonPhrase;
                    string result = response.Content.ReadAsStringAsync().Result;
                    throw new Exception(String.Format("Exception when calling service: {0} {1}", url, errorResponse + result));
                }
                return response.Content.ReadAsStringAsync().Result;
            }
        }

        /// <summary>
        /// Get certificate from local store
        /// </summary>
        /// <param name="thumbprint"></param>
        /// <returns></returns>
        public static X509Certificate2 GetCertificate(string thumbprint)
        {
            X509Store store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
            try
            {
                store.Open(OpenFlags.ReadOnly);
                X509Certificate2Collection certCollection = store.Certificates;
                X509Certificate2Collection currentCerts = certCollection;
                X509Certificate2Collection currentCert = currentCerts.Find(X509FindType.FindByThumbprint, thumbprint, false);
                if (currentCert.Count != 0)
                    return currentCert[0];
                else
                    return null;
            }
            catch (Exception ex)
            {
                return null;
            }
            finally
            {
                store.Close();
            }
        }

        /// <summary>
        /// Get api key
        /// </summary>
        /// <param name="user"></param>
        /// <param name="pass"></param>
        /// <returns></returns>
        public static string GetApiKey(string user, string pass)
        {
            return Convert.ToBase64String(Encoding.UTF8.GetBytes(user + ":" + pass));
        }
    }
}
